#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from typing import ClassVar
import uuid

from openstackclient.tests.functional.volume.v3 import common


class VolumeSnapshotTests(common.BaseVolumeTests):
    """Functional tests for volume snapshot."""

    VOLUME_NAME = uuid.uuid4().hex
    VOLUME_ID: ClassVar[str]

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # create a test volume used by all snapshot tests
        cmd_output = cls.openstack(
            'volume create ' + '--size 1 ' + cls.VOLUME_NAME,
            parse_output=True,
        )
        cls.wait_for_status('volume', cls.VOLUME_NAME, 'available')
        cls.VOLUME_ID = cmd_output['id']

    @classmethod
    def tearDownClass(cls):
        try:
            cls.wait_for_status('volume', cls.VOLUME_NAME, 'available')
            raw_output = cls.openstack(
                'volume delete --force ' + cls.VOLUME_NAME
            )
            cls.assertOutput('', raw_output)
        finally:
            super().tearDownClass()

    def test_volume_snapshot(self):
        # create volume snapshot
        name = uuid.uuid4().hex

        cmd_output = self.openstack(
            'volume snapshot create '
            + '--volume '
            + self.VOLUME_NAME
            + ' --description aaaa '
            + '--property Alpha=a '
            + name,
            parse_output=True,
        )
        snap_id = cmd_output['id']

        self.addCleanup(self.wait_for_delete, 'volume snapshot', snap_id)
        # delete volume snapshot
        self.addCleanup(
            self.openstack,
            'volume snapshot delete ' + snap_id,
        )
        self.wait_for_status('volume snapshot', snap_id, 'available')

        # show volume snapshot
        snapshot_info = self.openstack(
            'volume snapshot show ' + name,
            parse_output=True,
        )

        self.assertEqual(name, snapshot_info['name'])
        self.assertEqual('aaaa', snapshot_info["description"])
        self.assertEqual({'Alpha': 'a'}, snapshot_info["properties"])

        # list volume snapshot --name
        cmd_output = self.openstack(
            'volume snapshot list --name ' + name,
            parse_output=True,
        )
        names = [x['Name'] for x in cmd_output]
        self.assertIn(name, names)

        # list volume snapshot --volume
        cmd_output = self.openstack(
            'volume snapshot list ' + '--volume ' + self.VOLUME_NAME,
            parse_output=True,
        )
        names = [x["Name"] for x in cmd_output]
        self.assertIn(name, names)

        # set volume snapshot
        new_name = name + "_"
        raw_output = self.openstack(
            'volume snapshot set '
            + '--name '
            + new_name
            + ' --description bbbb '
            + '--property Alpha=c '
            + '--property Beta=b '
            + snap_id,
        )
        self.assertOutput('', raw_output)

        cmd_output = self.openstack(
            'volume snapshot show ' + new_name,
            parse_output=True,
        )
        self.assertEqual(
            new_name,
            cmd_output["name"],
        )
        self.assertEqual(
            'bbbb',
            cmd_output["description"],
        )
        self.assertEqual(
            {'Alpha': 'c', 'Beta': 'b'},
            cmd_output["properties"],
        )

        # unset volume snapshot
        raw_output = self.openstack(
            'volume snapshot unset ' + '--property Alpha ' + new_name,
        )
        self.assertOutput('', raw_output)

        cmd_output = self.openstack(
            'volume snapshot show ' + new_name,
            parse_output=True,
        )
        self.assertEqual(
            {'Beta': 'b'},
            cmd_output["properties"],
        )

        # set volume snapshot --no-property, --state error
        raw_output = self.openstack(
            'volume snapshot set '
            + '--no-property '
            + '--state error '
            + new_name,
        )
        self.assertOutput('', raw_output)

        cmd_output = self.openstack(
            'volume snapshot show ' + new_name,
            parse_output=True,
        )
        self.assertEqual({}, cmd_output["properties"])

        # list volume snapshot --long --status
        cmd_output = self.openstack(
            'volume snapshot list ' + '--long ' + '--status error',
            parse_output=True,
        )
        names = [x["Name"] for x in cmd_output]
        self.assertIn(new_name, names)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from osc_lib import exceptions

from openstackclient.network.v2 import network_flavor_profile
from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes_v3
from openstackclient.tests.unit.network.v2 import fakes as network_fakes


class TestFlavorProfile(network_fakes.TestNetworkV2):
    def setUp(self):
        super().setUp()

        # Get the DomainManager Mock
        self.domains_mock = self.identity_client.domains


class TestCreateFlavorProfile(TestFlavorProfile):
    domain = identity_fakes_v3.FakeDomain.create_one_domain()
    new_flavor_profile = network_fakes.create_one_service_profile()

    columns = (
        'description',
        'driver',
        'enabled',
        'id',
        'meta_info',
    )

    data = (
        new_flavor_profile.description,
        new_flavor_profile.driver,
        new_flavor_profile.is_enabled,
        new_flavor_profile.id,
        new_flavor_profile.meta_info,
    )

    def setUp(self):
        super().setUp()
        self.network_client.create_service_profile.return_value = (
            self.new_flavor_profile
        )

        # Get the command object to test
        self.cmd = network_flavor_profile.CreateNetworkFlavorProfile(
            self.app, None
        )

    def test_create_all_options(self):
        arglist = [
            "--description",
            self.new_flavor_profile.description,
            "--enable",
            "--driver",
            self.new_flavor_profile.driver,
            "--metainfo",
            self.new_flavor_profile.meta_info,
        ]

        verifylist = [
            ('description', self.new_flavor_profile.description),
            ('enable', True),
            ('driver', self.new_flavor_profile.driver),
            ('metainfo', self.new_flavor_profile.meta_info),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.create_service_profile.assert_called_once_with(
            **{
                'description': self.new_flavor_profile.description,
                'enabled': self.new_flavor_profile.is_enabled,
                'driver': self.new_flavor_profile.driver,
                'metainfo': self.new_flavor_profile.meta_info,
            }
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_with_metainfo(self):
        arglist = [
            "--description",
            self.new_flavor_profile.description,
            "--enable",
            "--metainfo",
            self.new_flavor_profile.meta_info,
        ]

        verifylist = [
            ('description', self.new_flavor_profile.description),
            ('enable', True),
            ('metainfo', self.new_flavor_profile.meta_info),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.create_service_profile.assert_called_once_with(
            **{
                'description': self.new_flavor_profile.description,
                'enabled': self.new_flavor_profile.is_enabled,
                'metainfo': self.new_flavor_profile.meta_info,
            }
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_with_driver(self):
        arglist = [
            "--description",
            self.new_flavor_profile.description,
            "--enable",
            "--driver",
            self.new_flavor_profile.driver,
        ]

        verifylist = [
            ('description', self.new_flavor_profile.description),
            ('enable', True),
            ('driver', self.new_flavor_profile.driver),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.create_service_profile.assert_called_once_with(
            **{
                'description': self.new_flavor_profile.description,
                'enabled': self.new_flavor_profile.is_enabled,
                'driver': self.new_flavor_profile.driver,
            }
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_without_driver_and_metainfo(self):
        arglist = [
            "--description",
            self.new_flavor_profile.description,
            "--enable",
        ]

        verifylist = [
            ('description', self.new_flavor_profile.description),
            ('enable', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args,
        )

    def test_create_disable(self):
        arglist = [
            '--disable',
            '--driver',
            self.new_flavor_profile.driver,
        ]
        verifylist = [
            ('disable', True),
            ('driver', self.new_flavor_profile.driver),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.create_service_profile.assert_called_once_with(
            **{
                'enabled': False,
                'driver': self.new_flavor_profile.driver,
            }
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteFlavorProfile(TestFlavorProfile):
    # The network flavor_profiles to delete.
    _network_flavor_profiles = network_fakes.create_service_profile(count=2)

    def setUp(self):
        super().setUp()
        self.network_client.delete_service_profile.return_value = None

        self.network_client.find_service_profile = (
            network_fakes.get_service_profile(
                flavor_profile=self._network_flavor_profiles
            )
        )

        # Get the command object to test
        self.cmd = network_flavor_profile.DeleteNetworkFlavorProfile(
            self.app, None
        )

    def test_network_flavor_profile_delete(self):
        arglist = [
            self._network_flavor_profiles[0].id,
        ]
        verifylist = [
            ('flavor_profile', [self._network_flavor_profiles[0].id]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.network_client.find_service_profile.assert_called_once_with(
            self._network_flavor_profiles[0].id, ignore_missing=False
        )
        self.network_client.delete_service_profile.assert_called_once_with(
            self._network_flavor_profiles[0]
        )
        self.assertIsNone(result)

    def test_multi_network_flavor_profiles_delete(self):
        arglist = []

        for a in self._network_flavor_profiles:
            arglist.append(a.id)
        verifylist = [
            ('flavor_profile', arglist),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        calls = []
        for a in self._network_flavor_profiles:
            calls.append(mock.call(a))
        self.network_client.delete_service_profile.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multi_network_flavor_profiles_delete_with_exception(self):
        arglist = [
            self._network_flavor_profiles[0].id,
            'unexist_network_flavor_profile',
        ]
        verifylist = [
            (
                'flavor_profile',
                [
                    self._network_flavor_profiles[0].id,
                    'unexist_network_flavor_profile',
                ],
            ),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [
            self._network_flavor_profiles[0],
            exceptions.CommandError,
        ]
        self.network_client.find_service_profile.side_effect = find_mock_result

        try:
            self.cmd.take_action(parsed_args)
            self.fail('CommandError should be raised.')
        except exceptions.CommandError as e:
            self.assertEqual(
                '1 of 2 flavor_profiles failed to delete.', str(e)
            )

        self.network_client.find_service_profile.assert_any_call(
            self._network_flavor_profiles[0].id, ignore_missing=False
        )
        self.network_client.find_service_profile.assert_any_call(
            'unexist_network_flavor_profile', ignore_missing=False
        )
        self.network_client.delete_service_profile.assert_called_once_with(
            self._network_flavor_profiles[0]
        )


class TestListFlavorProfile(TestFlavorProfile):
    # The network flavor profiles list
    _network_flavor_profiles = network_fakes.create_service_profile(count=2)

    columns = (
        'ID',
        'Driver',
        'Enabled',
        'Metainfo',
        'Description',
    )
    data = []
    for flavor_profile in _network_flavor_profiles:
        data.append(
            (
                flavor_profile.id,
                flavor_profile.driver,
                flavor_profile.is_enabled,
                flavor_profile.meta_info,
                flavor_profile.description,
            )
        )

    def setUp(self):
        super().setUp()
        self.network_client.service_profiles.return_value = (
            self._network_flavor_profiles
        )

        # Get the command object to test
        self.cmd = network_flavor_profile.ListNetworkFlavorProfile(
            self.app, None
        )

    def test_network_flavor_profile_list(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.service_profiles.assert_called_once_with(**{})
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowFlavorProfile(TestFlavorProfile):
    # The network flavor profile to show.
    network_flavor_profile = network_fakes.create_one_service_profile()
    columns = (
        'description',
        'driver',
        'enabled',
        'id',
        'meta_info',
    )
    data = (
        network_flavor_profile.description,
        network_flavor_profile.driver,
        network_flavor_profile.is_enabled,
        network_flavor_profile.id,
        network_flavor_profile.meta_info,
    )

    def setUp(self):
        super().setUp()
        self.network_client.find_service_profile.return_value = (
            self.network_flavor_profile
        )

        # Get the command object to test
        self.cmd = network_flavor_profile.ShowNetworkFlavorProfile(
            self.app, None
        )

    def test_show_all_options(self):
        arglist = [
            self.network_flavor_profile.id,
        ]
        verifylist = [
            ('flavor_profile', self.network_flavor_profile.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.find_service_profile.assert_called_once_with(
            self.network_flavor_profile.id, ignore_missing=False
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestSetFlavorProfile(TestFlavorProfile):
    # The network flavor profile to set.
    network_flavor_profile = network_fakes.create_one_service_profile()

    def setUp(self):
        super().setUp()
        self.network_client.update_service_profile.return_value = None

        self.network_client.find_service_profile.return_value = (
            self.network_flavor_profile
        )

        # Get the command object to test
        self.cmd = network_flavor_profile.SetNetworkFlavorProfile(
            self.app, None
        )

    def test_set_nothing(self):
        arglist = [self.network_flavor_profile.id]
        verifylist = [
            ('flavor_profile', self.network_flavor_profile.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        attrs = {}
        self.network_client.update_service_profile.assert_called_with(
            self.network_flavor_profile, **attrs
        )
        self.assertIsNone(result)

    def test_set_enable(self):
        arglist = [
            '--enable',
            self.network_flavor_profile.id,
        ]
        verifylist = [
            ('enable', True),
            ('flavor_profile', self.network_flavor_profile.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        attrs = {
            'enabled': True,
        }
        self.network_client.update_service_profile.assert_called_with(
            self.network_flavor_profile, **attrs
        )
        self.assertIsNone(result)

    def test_set_disable(self):
        arglist = [
            '--disable',
            self.network_flavor_profile.id,
        ]
        verifylist = [
            ('disable', True),
            ('flavor_profile', self.network_flavor_profile.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        attrs = {
            'enabled': False,
        }
        self.network_client.update_service_profile.assert_called_with(
            self.network_flavor_profile, **attrs
        )
        self.assertIsNone(result)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import operator
import uuid

from openstack.network.v2 import tap_mirror
from openstack.test import fakes as sdk_fakes
from osc_lib import utils as osc_utils
from osc_lib.utils import columns as column_util

from openstackclient.network.v2.taas import tap_mirror as osc_tap_mirror
from openstackclient.tests.unit.network.v2 import fakes as network_fakes


columns_long = tuple(
    col
    for col, _, listing_mode in osc_tap_mirror._attr_map
    if listing_mode in (column_util.LIST_BOTH, column_util.LIST_LONG_ONLY)
)
headers_long = tuple(
    head
    for _, head, listing_mode in osc_tap_mirror._attr_map
    if listing_mode in (column_util.LIST_BOTH, column_util.LIST_LONG_ONLY)
)
sorted_attr_map = sorted(osc_tap_mirror._attr_map, key=operator.itemgetter(1))
sorted_columns = tuple(col for col, _, _ in sorted_attr_map)
sorted_headers = tuple(head for _, head, _ in sorted_attr_map)


def _get_data(attrs, columns=sorted_columns):
    return osc_utils.get_dict_properties(attrs, columns)


class TestCreateTapMirror(network_fakes.TestNetworkV2):
    columns = (
        'description',
        'directions',
        'id',
        'mirror_type',
        'name',
        'port_id',
        'project_id',
        'remote_ip',
    )

    def setUp(self):
        super().setUp()
        self.cmd = osc_tap_mirror.CreateTapMirror(self.app, None)

    def test_create_tap_mirror(self):
        port_id = str(uuid.uuid4())
        fake_port = network_fakes.create_one_port(attrs={'id': port_id})
        fake_tap_mirror = sdk_fakes.generate_fake_resource(
            tap_mirror.TapMirror, **{'port_id': port_id, 'directions': 'IN=99'}
        )
        self.network_client.create_tap_mirror.return_value = fake_tap_mirror
        self.network_client.find_port.return_value = fake_port
        self.network_client.find_tap_mirror.side_effect = (
            lambda _, name_or_id: {'id': name_or_id}
        )
        arg_list = [
            '--name',
            fake_tap_mirror['name'],
            '--port',
            fake_tap_mirror['port_id'],
            '--directions',
            fake_tap_mirror['directions'],
            '--remote-ip',
            fake_tap_mirror['remote_ip'],
            '--mirror-type',
            fake_tap_mirror['mirror_type'],
        ]

        verify_directions = fake_tap_mirror['directions'].split('=')
        verify_directions_dict = {verify_directions[0]: verify_directions[1]}

        verify_list = [
            ('name', fake_tap_mirror['name']),
            ('port_id', fake_tap_mirror['port_id']),
            ('directions', verify_directions_dict),
            ('remote_ip', fake_tap_mirror['remote_ip']),
            ('mirror_type', fake_tap_mirror['mirror_type']),
        ]

        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)
        self.network_client.find_tap_mirror.return_value = fake_tap_mirror

        columns, data = self.cmd.take_action(parsed_args)
        self.network_client.create_tap_mirror.assert_called_once_with(
            **{
                'name': fake_tap_mirror['name'],
                'port_id': fake_tap_mirror['port_id'],
                'directions': verify_directions_dict,
                'remote_ip': fake_tap_mirror['remote_ip'],
                'mirror_type': fake_tap_mirror['mirror_type'],
            }
        )
        self.assertEqual(self.columns, columns)
        fake_data = _get_data(
            fake_tap_mirror, osc_tap_mirror._get_columns(fake_tap_mirror)[1]
        )
        self.assertEqual(fake_data, data)


class TestListTapMirror(network_fakes.TestNetworkV2):
    def setUp(self):
        super().setUp()
        self.cmd = osc_tap_mirror.ListTapMirror(self.app, None)

    def test_list_tap_mirror(self):
        """Test List Tap Mirror."""
        fake_tap_mirrors = list(
            sdk_fakes.generate_fake_resources(tap_mirror.TapMirror, count=4)
        )
        self.network_client.tap_mirrors.return_value = fake_tap_mirrors

        arg_list = []
        verify_list = []

        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)

        headers, data = self.cmd.take_action(parsed_args)

        self.network_client.tap_mirrors.assert_called_once()
        self.assertEqual(headers, list(headers_long))
        self.assertCountEqual(
            list(data),
            [
                _get_data(fake_tap_mirror, columns_long)
                for fake_tap_mirror in fake_tap_mirrors
            ],
        )


class TestDeleteTapMirror(network_fakes.TestNetworkV2):
    def setUp(self):
        super().setUp()
        self.network_client.find_tap_mirror.side_effect = (
            lambda name_or_id, ignore_missing: tap_mirror.TapMirror(
                id=name_or_id
            )
        )
        self.cmd = osc_tap_mirror.DeleteTapMirror(self.app, None)

    def test_delete_tap_mirror(self):
        """Test Delete Tap Mirror."""

        fake_tap_mirror = sdk_fakes.generate_fake_resource(
            tap_mirror.TapMirror
        )

        arg_list = [
            fake_tap_mirror['id'],
        ]
        verify_list = [
            (osc_tap_mirror.TAP_MIRROR, [fake_tap_mirror['id']]),
        ]

        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)
        result = self.cmd.take_action(parsed_args)

        self.network_client.delete_tap_mirror.assert_called_once_with(
            fake_tap_mirror['id']
        )
        self.assertIsNone(result)


class TestShowTapMirror(network_fakes.TestNetworkV2):
    columns = (
        'description',
        'directions',
        'id',
        'mirror_type',
        'name',
        'port_id',
        'project_id',
        'remote_ip',
    )

    def setUp(self):
        super().setUp()
        self.network_client.find_tap_mirror.side_effect = (
            lambda name_or_id, ignore_missing: tap_mirror.TapMirror(
                id=name_or_id
            )
        )
        self.cmd = osc_tap_mirror.ShowTapMirror(self.app, None)

    def test_show_tap_mirror(self):
        """Test Show Tap Mirror."""

        fake_tap_mirror = sdk_fakes.generate_fake_resource(
            tap_mirror.TapMirror
        )
        self.network_client.get_tap_mirror.return_value = fake_tap_mirror
        arg_list = [
            fake_tap_mirror['id'],
        ]
        verify_list = [
            (osc_tap_mirror.TAP_MIRROR, fake_tap_mirror['id']),
        ]

        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)

        headers, data = self.cmd.take_action(parsed_args)

        self.network_client.get_tap_mirror.assert_called_once_with(
            fake_tap_mirror['id']
        )
        self.assertEqual(self.columns, headers)
        fake_data = _get_data(
            fake_tap_mirror, osc_tap_mirror._get_columns(fake_tap_mirror)[1]
        )
        self.assertEqual(fake_data, data)


class TestUpdateTapMirror(network_fakes.TestNetworkV2):
    _new_name = 'new_name'
    columns = (
        'description',
        'directions',
        'id',
        'mirror_type',
        'name',
        'port_id',
        'project_id',
        'remote_ip',
    )

    def setUp(self):
        super().setUp()
        self.cmd = osc_tap_mirror.UpdateTapMirror(self.app, None)
        self.network_client.find_tap_mirror.side_effect = (
            lambda name_or_id, ignore_missing: tap_mirror.TapMirror(
                id=name_or_id
            )
        )

    def test_update_tap_mirror(self):
        """Test update Tap Mirror"""
        fake_tap_mirror = sdk_fakes.generate_fake_resource(
            tap_mirror.TapMirror
        )
        new_tap_mirror = copy.deepcopy(fake_tap_mirror)
        new_tap_mirror['name'] = self._new_name

        self.network_client.update_tap_mirror.return_value = new_tap_mirror

        arg_list = [
            fake_tap_mirror['id'],
            '--name',
            self._new_name,
        ]
        verify_list = [('name', self._new_name)]

        parsed_args = self.check_parser(self.cmd, arg_list, verify_list)
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {'name': self._new_name}

        self.network_client.update_tap_mirror.assert_called_once_with(
            fake_tap_mirror['id'], **attrs
        )
        self.assertEqual(self.columns, columns)
        fake_data = _get_data(
            new_tap_mirror, osc_tap_mirror._get_columns(new_tap_mirror)[1]
        )
        self.assertEqual(fake_data, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Address scope action implementations"""

import argparse
import logging
from collections.abc import Iterable, Sequence
from typing import Any

from osc_lib import exceptions
from osc_lib import utils

from openstackclient import command
from openstackclient.i18n import _
from openstackclient.identity import common as identity_common
from openstackclient.network import common

LOG = logging.getLogger(__name__)


def _get_columns(item: Any) -> tuple[tuple[str, ...], tuple[str, ...]]:
    column_map = {
        'is_shared': 'shared',
    }
    hidden_columns = ['location', 'tenant_id']
    return utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden_columns
    )


def _get_attrs(
    client_manager: Any, parsed_args: argparse.Namespace
) -> dict[str, Any]:
    attrs: dict[str, Any] = {}
    attrs['name'] = parsed_args.name
    attrs['ip_version'] = parsed_args.ip_version
    if parsed_args.share:
        attrs['shared'] = True
    if parsed_args.no_share:
        attrs['shared'] = False
    if 'project' in parsed_args and parsed_args.project is not None:
        identity_client = client_manager.identity
        project_id = identity_common.find_project(
            identity_client,
            parsed_args.project,
            parsed_args.project_domain,
        ).id
        attrs['project_id'] = project_id

    return attrs


# TODO(rtheis): Use the SDK resource mapped attribute names once the
# OSC minimum requirements include SDK 1.0.
class CreateAddressScope(command.ShowOne, common.NeutronCommandWithExtraArgs):
    _description = _("Create a new Address Scope")

    def get_parser(self, prog_name: str) -> argparse.ArgumentParser:
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'name', metavar="<name>", help=_("New address scope name")
        )
        parser.add_argument(
            '--ip-version',
            type=int,
            default=4,
            choices=[4, 6],
            help=_("IP version (default is 4)"),
        )
        parser.add_argument(
            '--project',
            metavar="<project>",
            help=_("Owner's project (name or ID)"),
        )
        identity_common.add_project_domain_option_to_parser(parser)

        share_group = parser.add_mutually_exclusive_group()
        share_group.add_argument(
            '--share',
            action='store_true',
            help=_('Share the address scope between projects'),
        )
        share_group.add_argument(
            '--no-share',
            action='store_true',
            help=_(
                'Do not share the address scope between projects (default)'
            ),
        )

        return parser

    def take_action(
        self, parsed_args: argparse.Namespace
    ) -> tuple[Sequence[str], Iterable[Any]]:
        client = self.app.client_manager.network
        attrs = _get_attrs(self.app.client_manager, parsed_args)
        attrs.update(
            self._parse_extra_properties(parsed_args.extra_properties)
        )
        obj = client.create_address_scope(**attrs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters={})

        return (display_columns, data)


class DeleteAddressScope(command.Command):
    _description = _("Delete address scope(s)")

    def get_parser(self, prog_name: str) -> argparse.ArgumentParser:
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'address_scope',
            metavar="<address-scope>",
            nargs='+',
            help=_("Address scope(s) to delete (name or ID)"),
        )

        return parser

    def take_action(self, parsed_args: argparse.Namespace) -> None:
        client = self.app.client_manager.network
        result = 0

        for scope in parsed_args.address_scope:
            try:
                obj = client.find_address_scope(scope, ignore_missing=False)
                client.delete_address_scope(obj)
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        "Failed to delete address scope with "
                        "name or ID '%(scope)s': %(e)s"
                    ),
                    {'scope': scope, 'e': e},
                )

        if result > 0:
            total = len(parsed_args.address_scope)
            msg = _(
                "%(result)s of %(total)s address scopes failed to delete."
            ) % {'result': result, 'total': total}
            raise exceptions.CommandError(msg)


# TODO(yanxing'an): Use the SDK resource mapped attribute names once the
# OSC minimum requirements include SDK 1.0.
class ListAddressScope(command.Lister):
    _description = _("List address scopes")

    def get_parser(self, prog_name: str) -> argparse.ArgumentParser:
        parser = super().get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("List only address scopes with the specified name"),
        )
        parser.add_argument(
            '--ip-version',
            type=int,
            choices=[4, 6],
            metavar='<ip-version>',
            dest='ip_version',
            help=_(
                "List only address scopes with the specified IP version "
                "networks (4 or 6)"
            ),
        )
        parser.add_argument(
            '--project',
            metavar="<project>",
            help=_(
                "List only address scopes with the specified project "
                "(name or ID)"
            ),
        )
        identity_common.add_project_domain_option_to_parser(parser)

        shared_group = parser.add_mutually_exclusive_group()
        shared_group.add_argument(
            '--share',
            action='store_true',
            help=_("List only address scopes shared between projects"),
        )
        shared_group.add_argument(
            '--no-share',
            action='store_true',
            help=_("List only address scopes not shared between projects"),
        )
        return parser

    def take_action(
        self, parsed_args: argparse.Namespace
    ) -> tuple[tuple[str, ...], Iterable[tuple[Any, ...]]]:
        client = self.app.client_manager.network
        columns = (
            'id',
            'name',
            'ip_version',
            'is_shared',
            'project_id',
        )
        column_headers = (
            'ID',
            'Name',
            'IP Version',
            'Shared',
            'Project',
        )
        attrs = {}
        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.ip_version:
            attrs['ip_version'] = parsed_args.ip_version
        if parsed_args.share:
            attrs['is_shared'] = True
        if parsed_args.no_share:
            attrs['is_shared'] = False
        if 'project' in parsed_args and parsed_args.project is not None:
            identity_client = self.app.client_manager.identity
            project_id = identity_common.find_project(
                identity_client,
                parsed_args.project,
                parsed_args.project_domain,
            ).id
            attrs['project_id'] = project_id
        data = client.address_scopes(**attrs)

        return (
            column_headers,
            (
                utils.get_item_properties(
                    s,
                    columns,
                    formatters={},
                )
                for s in data
            ),
        )


# TODO(rtheis): Use the SDK resource mapped attribute names once the
# OSC minimum requirements include SDK 1.0.
class SetAddressScope(common.NeutronCommandWithExtraArgs):
    _description = _("Set address scope properties")

    def get_parser(self, prog_name: str) -> argparse.ArgumentParser:
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'address_scope',
            metavar="<address-scope>",
            help=_("Address scope to modify (name or ID)"),
        )
        parser.add_argument(
            '--name', metavar="<name>", help=_('Set address scope name')
        )
        share_group = parser.add_mutually_exclusive_group()
        share_group.add_argument(
            '--share',
            action='store_true',
            help=_('Share the address scope between projects'),
        )
        share_group.add_argument(
            '--no-share',
            action='store_true',
            help=_('Do not share the address scope between projects'),
        )

        return parser

    def take_action(self, parsed_args: argparse.Namespace) -> None:
        client = self.app.client_manager.network
        obj = client.find_address_scope(
            parsed_args.address_scope, ignore_missing=False
        )
        attrs = {}
        if parsed_args.name is not None:
            attrs['name'] = parsed_args.name
        if parsed_args.share:
            attrs['shared'] = True
        if parsed_args.no_share:
            attrs['shared'] = False
        attrs.update(
            self._parse_extra_properties(parsed_args.extra_properties)
        )
        client.update_address_scope(obj, **attrs)


class ShowAddressScope(command.ShowOne):
    _description = _("Display address scope details")

    def get_parser(self, prog_name: str) -> argparse.ArgumentParser:
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'address_scope',
            metavar="<address-scope>",
            help=_("Address scope to display (name or ID)"),
        )

        return parser

    def take_action(
        self, parsed_args: argparse.Namespace
    ) -> tuple[Sequence[str], Iterable[Any]]:
        client = self.app.client_manager.network
        obj = client.find_address_scope(
            parsed_args.address_scope, ignore_missing=False
        )
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters={})

        return (display_columns, data)



# Copyright (c) 2019, Intel Corporation.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from unittest import mock
from unittest.mock import call

from osc_lib import exceptions

from openstackclient.network.v2 import network_segment_range
from openstackclient.tests.unit.network.v2 import fakes as network_fakes
from openstackclient.tests.unit import utils as tests_utils


class TestAuxiliaryFunctions(tests_utils.TestCase):
    def test__get_ranges(self):
        input_reference = [
            ([1, 2, 3, 4, 5, 6, 7], ['1-7']),
            ([1, 2, 5, 4, 3, 6, 7], ['1-7']),
            ([1, 2, 4, 3, 7, 6], ['1-4', '6-7']),
            ([1, 2, 4, 3, '13', 12, '7', '6'], ['1-4', '6-7', '12-13']),
        ]
        for input, reference in input_reference:
            self.assertEqual(
                reference, list(network_segment_range._get_ranges(input))
            )


class TestNetworkSegmentRange(network_fakes.TestNetworkV2):
    def setUp(self):
        super().setUp()


class TestCreateNetworkSegmentRange(TestNetworkSegmentRange):
    # The network segment range to create.
    _network_segment_range = network_fakes.create_one_network_segment_range()

    columns = (
        'available',
        'default',
        'id',
        'maximum',
        'minimum',
        'name',
        'network_type',
        'physical_network',
        'project_id',
        'shared',
        'used',
    )

    data = (
        ['100-103', '105'],
        _network_segment_range.default,
        _network_segment_range.id,
        _network_segment_range.maximum,
        _network_segment_range.minimum,
        _network_segment_range.name,
        _network_segment_range.network_type,
        _network_segment_range.physical_network,
        _network_segment_range.project_id,
        _network_segment_range.shared,
        {'3312e4ba67864b2eb53f3f41432f8efc': ['104', '106']},
    )

    def setUp(self):
        super().setUp()

        self.network_client.create_network_segment_range.return_value = (
            self._network_segment_range
        )

        # Get the command object to test
        self.cmd = network_segment_range.CreateNetworkSegmentRange(
            self.app, None
        )

    def test_create_no_options(self):
        # Missing required args should bail here
        self.assertRaises(
            tests_utils.ParserException, self.check_parser, self.cmd, [], []
        )

    def test_create_invalid_network_type(self):
        arglist = [
            '--private',
            '--project',
            self._network_segment_range.project_id,
            '--network-type',
            'foo',
            '--minimum',
            str(self._network_segment_range.minimum),
            '--maximum',
            str(self._network_segment_range.maximum),
            self._network_segment_range.name,
        ]
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            [],
        )

    def test_create_default_with_project_id(self):
        arglist = [
            '--project',
            self._network_segment_range.project_id,
            '--network-type',
            'vxlan',
            '--minimum',
            str(self._network_segment_range.minimum),
            '--maximum',
            str(self._network_segment_range.maximum),
            self._network_segment_range.name,
        ]
        verifylist = [
            ('project', self._network_segment_range.project_id),
            ('network_type', 'vxlan'),
            ('minimum', self._network_segment_range.minimum),
            ('maximum', self._network_segment_range.maximum),
            ('name', self._network_segment_range.name),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args
        )

    def test_create_shared_with_project_id(self):
        arglist = [
            '--shared',
            '--project',
            self._network_segment_range.project_id,
            '--network-type',
            'vxlan',
            '--minimum',
            str(self._network_segment_range.minimum),
            '--maximum',
            str(self._network_segment_range.maximum),
            self._network_segment_range.name,
        ]
        verifylist = [
            ('shared', True),
            ('project', self._network_segment_range.project_id),
            ('network_type', 'vxlan'),
            ('minimum', self._network_segment_range.minimum),
            ('maximum', self._network_segment_range.maximum),
            ('name', self._network_segment_range.name),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args
        )

    def test_create_tunnel_with_physical_network(self):
        arglist = [
            '--shared',
            '--network-type',
            'vxlan',
            '--physical-network',
            self._network_segment_range.physical_network,
            '--minimum',
            str(self._network_segment_range.minimum),
            '--maximum',
            str(self._network_segment_range.maximum),
            self._network_segment_range.name,
        ]
        verifylist = [
            ('shared', True),
            ('network_type', 'vxlan'),
            ('physical_network', self._network_segment_range.physical_network),
            ('minimum', self._network_segment_range.minimum),
            ('maximum', self._network_segment_range.maximum),
            ('name', self._network_segment_range.name),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args
        )

    def test_create_minimum_options(self):
        arglist = [
            '--network-type',
            'vxlan',
            '--minimum',
            str(self._network_segment_range.minimum),
            '--maximum',
            str(self._network_segment_range.maximum),
            self._network_segment_range.name,
        ]
        verifylist = [
            ('network_type', 'vxlan'),
            ('minimum', self._network_segment_range.minimum),
            ('maximum', self._network_segment_range.maximum),
            ('name', self._network_segment_range.name),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.create_network_segment_range.assert_called_once_with(
            **{
                'shared': True,
                'network_type': 'vxlan',
                'minimum': self._network_segment_range.minimum,
                'maximum': self._network_segment_range.maximum,
                'name': self._network_segment_range.name,
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_private_minimum_options(self):
        arglist = [
            '--private',
            '--project',
            self._network_segment_range.project_id,
            '--network-type',
            'vxlan',
            '--minimum',
            str(self._network_segment_range.minimum),
            '--maximum',
            str(self._network_segment_range.maximum),
            self._network_segment_range.name,
        ]
        verifylist = [
            ('shared', False),
            ('project', self._network_segment_range.project_id),
            ('network_type', 'vxlan'),
            ('minimum', self._network_segment_range.minimum),
            ('maximum', self._network_segment_range.maximum),
            ('name', self._network_segment_range.name),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.create_network_segment_range.assert_called_once_with(
            **{
                'shared': False,
                'project_id': mock.ANY,
                'network_type': 'vxlan',
                'minimum': self._network_segment_range.minimum,
                'maximum': self._network_segment_range.maximum,
                'name': self._network_segment_range.name,
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_shared_minimum_options(self):
        arglist = [
            '--shared',
            '--network-type',
            'vxlan',
            '--minimum',
            str(self._network_segment_range.minimum),
            '--maximum',
            str(self._network_segment_range.maximum),
            self._network_segment_range.name,
        ]
        verifylist = [
            ('shared', True),
            ('network_type', 'vxlan'),
            ('minimum', self._network_segment_range.minimum),
            ('maximum', self._network_segment_range.maximum),
            ('name', self._network_segment_range.name),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.create_network_segment_range.assert_called_once_with(
            **{
                'shared': True,
                'network_type': 'vxlan',
                'minimum': self._network_segment_range.minimum,
                'maximum': self._network_segment_range.maximum,
                'name': self._network_segment_range.name,
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_all_options(self):
        arglist = [
            '--private',
            '--project',
            self._network_segment_range.project_id,
            '--network-type',
            self._network_segment_range.network_type,
            '--physical-network',
            self._network_segment_range.physical_network,
            '--minimum',
            str(self._network_segment_range.minimum),
            '--maximum',
            str(self._network_segment_range.maximum),
            self._network_segment_range.name,
        ]
        verifylist = [
            ('shared', self._network_segment_range.shared),
            ('project', self._network_segment_range.project_id),
            ('network_type', self._network_segment_range.network_type),
            ('physical_network', self._network_segment_range.physical_network),
            ('minimum', self._network_segment_range.minimum),
            ('maximum', self._network_segment_range.maximum),
            ('name', self._network_segment_range.name),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.create_network_segment_range.assert_called_once_with(
            **{
                'shared': self._network_segment_range.shared,
                'project_id': mock.ANY,
                'network_type': self._network_segment_range.network_type,
                'physical_network': self._network_segment_range.physical_network,
                'minimum': self._network_segment_range.minimum,
                'maximum': self._network_segment_range.maximum,
                'name': self._network_segment_range.name,
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteNetworkSegmentRange(TestNetworkSegmentRange):
    # The network segment ranges to delete.
    _network_segment_ranges = network_fakes.create_network_segment_ranges()

    def setUp(self):
        super().setUp()

        self.network_client.delete_network_segment_range.return_value = None

        self.network_client.find_network_segment_range.side_effect = (
            self._network_segment_ranges
        )

        # Get the command object to test
        self.cmd = network_segment_range.DeleteNetworkSegmentRange(
            self.app, None
        )

    def test_delete(self):
        arglist = [
            self._network_segment_ranges[0].id,
        ]
        verifylist = [
            ('network_segment_range', [self._network_segment_ranges[0].id]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.network_client.delete_network_segment_range.assert_called_once_with(
            self._network_segment_ranges[0]
        )
        self.assertIsNone(result)

    def test_delete_multiple(self):
        arglist = []
        for _network_segment_range in self._network_segment_ranges:
            arglist.append(_network_segment_range.id)
        verifylist = [
            ('network_segment_range', arglist),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        calls = []
        for _network_segment_range in self._network_segment_ranges:
            calls.append(call(_network_segment_range))
        self.network_client.delete_network_segment_range.assert_has_calls(
            calls
        )
        self.assertIsNone(result)

    def test_delete_multiple_with_exception(self):
        arglist = [self._network_segment_ranges[0].id, 'doesnotexist']
        verifylist = [
            (
                'network_segment_range',
                [self._network_segment_ranges[0].id, 'doesnotexist'],
            ),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [
            self._network_segment_ranges[0],
            exceptions.CommandError,
        ]
        self.network_client.find_network_segment_range.side_effect = (
            find_mock_result
        )

        try:
            self.cmd.take_action(parsed_args)
            self.fail('CommandError should be raised.')
        except exceptions.CommandError as e:
            self.assertEqual(
                '1 of 2 network segment ranges failed to delete.', str(e)
            )

        self.network_client.find_network_segment_range.assert_any_call(
            self._network_segment_ranges[0].id, ignore_missing=False
        )
        self.network_client.find_network_segment_range.assert_any_call(
            'doesnotexist', ignore_missing=False
        )
        self.network_client.delete_network_segment_range.assert_called_once_with(
            self._network_segment_ranges[0]
        )


class TestListNetworkSegmentRange(TestNetworkSegmentRange):
    _network_segment_ranges = network_fakes.create_network_segment_ranges(
        count=3
    )

    columns = (
        'ID',
        'Name',
        'Default',
        'Shared',
        'Project ID',
        'Network Type',
        'Physical Network',
        'Minimum ID',
        'Maximum ID',
    )
    columns_long = (*columns, 'Used', 'Available')

    data = []
    for _network_segment_range in _network_segment_ranges:
        data.append(
            (
                _network_segment_range.id,
                _network_segment_range.name,
                _network_segment_range.default,
                _network_segment_range.shared,
                _network_segment_range.project_id,
                _network_segment_range.network_type,
                _network_segment_range.physical_network,
                _network_segment_range.minimum,
                _network_segment_range.maximum,
            )
        )

    data_long = []
    for _network_segment_range in _network_segment_ranges:
        data_long.append(
            (
                _network_segment_range.id,
                _network_segment_range.name,
                _network_segment_range.default,
                _network_segment_range.shared,
                _network_segment_range.project_id,
                _network_segment_range.network_type,
                _network_segment_range.physical_network,
                _network_segment_range.minimum,
                _network_segment_range.maximum,
                {'3312e4ba67864b2eb53f3f41432f8efc': ['104', '106']},
                ['100-103', '105'],
            )
        )

    def setUp(self):
        super().setUp()

        self.network_client.network_segment_ranges.return_value = (
            self._network_segment_ranges
        )

        # Get the command object to test
        self.cmd = network_segment_range.ListNetworkSegmentRange(
            self.app, None
        )

    def test_list_no_option(self):
        arglist = []
        verifylist = [
            ('long', False),
            ('available', False),
            ('unavailable', False),
            ('used', False),
            ('unused', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.network_segment_ranges.assert_called_once_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_long(self):
        arglist = [
            '--long',
        ]
        verifylist = [
            ('long', True),
            ('available', False),
            ('unavailable', False),
            ('used', False),
            ('unused', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.network_segment_ranges.assert_called_once_with()
        self.assertEqual(self.columns_long, columns)
        self.assertEqual(self.data_long, list(data))


class TestSetNetworkSegmentRange(TestNetworkSegmentRange):
    # The network segment range to set.
    _network_segment_range = network_fakes.create_one_network_segment_range()
    # The network segment range updated.
    minimum_updated = _network_segment_range.minimum - 5
    maximum_updated = _network_segment_range.maximum + 5
    available_updated = [
        *list(range(minimum_updated, 104)),
        105,
        *list(range(107, maximum_updated + 1)),
    ]
    _network_segment_range_updated = (
        network_fakes.create_one_network_segment_range(
            attrs={
                'minimum': minimum_updated,
                'maximum': maximum_updated,
                'used': {
                    104: '3312e4ba67864b2eb53f3f41432f8efc',
                    106: '3312e4ba67864b2eb53f3f41432f8efc',
                },
                'available': available_updated,
            }
        )
    )

    def setUp(self):
        super().setUp()

        self.network_client.find_network_segment_range.return_value = (
            self._network_segment_range
        )

        # Get the command object to test
        self.cmd = network_segment_range.SetNetworkSegmentRange(self.app, None)

    def test_set_no_options(self):
        arglist = [
            self._network_segment_range.id,
        ]
        verifylist = [
            ('network_segment_range', self._network_segment_range.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.network_client.update_network_segment_range.return_value = (
            self._network_segment_range
        )

        result = self.cmd.take_action(parsed_args)

        self.network_client.update_network_segment_range.assert_called_once_with(
            self._network_segment_range, **{}
        )
        self.assertIsNone(result)

    def test_set_all_options(self):
        arglist = [
            '--name',
            'new name',
            '--minimum',
            str(self.minimum_updated),
            '--maximum',
            str(self.maximum_updated),
            self._network_segment_range.id,
        ]
        verifylist = [
            ('name', 'new name'),
            ('minimum', self.minimum_updated),
            ('maximum', self.maximum_updated),
            ('network_segment_range', self._network_segment_range.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.network_client.update_network_segment_range.return_value = (
            self._network_segment_range_updated
        )

        result = self.cmd.take_action(parsed_args)

        attrs = {
            'name': 'new name',
            'minimum': self.minimum_updated,
            'maximum': self.maximum_updated,
        }
        self.network_client.update_network_segment_range.assert_called_once_with(
            self._network_segment_range, **attrs
        )
        self.assertIsNone(result)


class TestShowNetworkSegmentRange(TestNetworkSegmentRange):
    # The network segment range to show.
    _network_segment_range = network_fakes.create_one_network_segment_range()

    columns = (
        'available',
        'default',
        'id',
        'maximum',
        'minimum',
        'name',
        'network_type',
        'physical_network',
        'project_id',
        'shared',
        'used',
    )

    data = (
        ['100-103', '105'],
        _network_segment_range.default,
        _network_segment_range.id,
        _network_segment_range.maximum,
        _network_segment_range.minimum,
        _network_segment_range.name,
        _network_segment_range.network_type,
        _network_segment_range.physical_network,
        _network_segment_range.project_id,
        _network_segment_range.shared,
        {'3312e4ba67864b2eb53f3f41432f8efc': ['104', '106']},
    )

    def setUp(self):
        super().setUp()

        self.network_client.find_network_segment_range.return_value = (
            self._network_segment_range
        )

        # Get the command object to test
        self.cmd = network_segment_range.ShowNetworkSegmentRange(
            self.app, None
        )

    def test_show_no_options(self):
        # Missing required args should bail here
        self.assertRaises(
            tests_utils.ParserException, self.check_parser, self.cmd, [], []
        )

    def test_show_all_options(self):
        arglist = [
            self._network_segment_range.id,
        ]
        verifylist = [
            ('network_segment_range', self._network_segment_range.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.network_client.find_network_segment_range.assert_called_once_with(
            self._network_segment_range.id, ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)
